@@ -2,16 +2,22 @@
 #' 
 #' Trains a causal forest that can be used to estimate
 #' conditional average treatment effects tau(X). When
-#' the treatment assignmnet W is binary and unconfounded,
+#' the treatment assignment W is binary and unconfounded,
 #' we have tau(X) = E[Y(1) - Y(0) | X = x], where Y(0) and
 #' Y(1) are potential outcomes corresponding to the two possible
 #' treatment states. When W is continuous, we effectively estimate
-#' an average partical effect Cov[Y, W | X = x] / Var[W | X = x],
+#' an average partial effect Cov[Y, W | X = x] / Var[W | X = x],
 #' and interpret it as a treatment effect given unconfoundedness.
 #'
 #' @param X The covariates used in the causal regression.
 #' @param Y The outcome.
 #' @param W The treatment assignment (may be binary or real).
+#' @param Y.hat Estimates of the expected responses E[Y | Xi], marginalizing
+#'              over treatment. If Y.hat = NULL, these are estimated using
+#'              a separate regression forest. See section 6.1.1 of the GRF paper for
+#'              further discussion of this quantity.
+#' @param W.hat Estimates of the treatment propensities E[W | Xi]. If W.hat = NULL,
+#'              these are estimated using a separate regression forest.
 #' @param sample.fraction Fraction of the data used to build each tree.
 #'                        Note: If honesty is used, these subsamples will
 #'                        further be cut in half.
@@ -27,10 +33,6 @@
 #' @param ci.group.size The forest will grow ci.group.size trees on each subsample.
 #'                      In order to provide confidence intervals, ci.group.size must
 #'                      be at least 2.
-#' @param precompute.nuisance Should we first run regression forests to estimate
-#'                            y(x) = E[Y|X=x] and w(x) = E[W|X=x], and then run a
-#'                            causal forest on the residuals? This approach is
-#'                            recommended, computational resources permitting.
 #' @param alpha A tuning parameter that controls the maximum imbalance of a split.
 #' @param imbalance.penalty A tuning parameter that controls how harshly imbalanced splits are penalized.
 #' @param stabilize.splits Whether or not the treatment should be taken into account when
@@ -68,10 +70,40 @@
 #' # Predict with confidence intervals; growing more trees is now recommended.
 #' c.forest = causal_forest(X, Y, W, num.trees = 4000)
 #' c.pred = predict(c.forest, X.test, estimate.variance = TRUE)
+#'
+#' # In some examples, pre-fitting models for Y and W separately may
+#' # be helpful (e.g., if different models use different covariates).
+#' # In some applications, one may even want to get Y.hat and W.hat
+#' # using a completely different method (e.g., boosting).
+#' n = 2000; p = 20
+#' X = matrix(rnorm(n * p), n, p)
+#' TAU = 1 / (1 + exp(-X[, 3]))
+#' W = rbinom(n, 1, 1 / (1 + exp(-X[, 1] - X[, 2])))
+#' Y = pmax(X[, 2] + X[, 3], 0) + rowMeans(X[, 4:6]) / 2 + W * TAU + rnorm(n)
+#'
+#' forest.W = regression_forest(X, W, tune.parameters = TRUE)
+#' W.hat = predict(forest.W)$predictions
+#'
+#' forest.Y = regression_forest(X, Y, tune.parameters = TRUE)
+#' Y.hat = predict(forest.Y)$predictions
+#'
+#' forest.Y.varimp = variable_importance(forest.Y)
+#'
+#' # Note: Forests may have a hard time when trained on very few variables
+#' # (e.g., ncol(X) = 1, 2, or 3). We recommend not being too aggressive
+#' # in selection.
+#' selected.vars = which(forest.Y.varimp / mean(forest.Y.varimp) > 0.2)
+#'
+#' tau.forest = causal_forest(X[,selected.vars], Y, W,
+#'                            W.hat = W.hat, Y.hat = Y.hat,
+#'                            tune.parameters = TRUE)
+#' tau.hat = predict(tau.forest)$predictions
 #' }
 #'
 #' @export
 causal_forest <- function(X, Y, W,
+                          Y.hat = NULL,
+                          W.hat = NULL,
                           sample.fraction = 0.5,
                           mtry = NULL,
                           num.trees = 2000,
@@ -79,7 +111,6 @@ causal_forest <- function(X, Y, W,
                           min.node.size = NULL,
                           honesty = TRUE,
                           ci.group.size = 2,
-                          precompute.nuisance = TRUE,
                           alpha = NULL,
                           imbalance.penalty = NULL,
                           stabilize.splits = TRUE,
@@ -87,8 +118,8 @@ causal_forest <- function(X, Y, W,
                           clusters = NULL,
                           samples_per_cluster = NULL,
                           tune.parameters = FALSE,
-                          num.fit.trees = 40,
-                          num.fit.reps = 100,
+                          num.fit.trees = 200,
+                          num.fit.reps = 50,
                           num.optimize.reps = 1000) {
     validate_X(X)
     if(length(Y) != nrow(X)) { stop("Y has incorrect length.") }
@@ -101,21 +132,28 @@ causal_forest <- function(X, Y, W,
     
     reduced.form.weight <- 0
 
-    if (!precompute.nuisance) {
-      Y.hat = 0
-      W.hat = 0
-    } else {
-      forest.Y <- regression_forest(X, Y, sample.fraction = sample.fraction, mtry = mtry, 
+    if (is.null(Y.hat)) {
+      forest.Y <- regression_forest(X, Y, sample.fraction = sample.fraction, mtry = mtry, tune.parameters = tune.parameters,
                                     num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, 
                                     honesty = TRUE, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
                                     clusters = clusters, samples_per_cluster = samples_per_cluster);
       Y.hat <- predict(forest.Y)$predictions
+    } else if (length(Y.hat) == 1) {
+      Y.hat <- rep(Y.hat, nrow(X))
+    } else if (length(Y.hat) != nrow(X)) {
+      stop("Y.hat has incorrect length.")
+    }
 
-      forest.W <- regression_forest(X, W, sample.fraction = sample.fraction, mtry = mtry, 
+    if (is.null(W.hat)) {
+      forest.W <- regression_forest(X, W, sample.fraction = sample.fraction, mtry = mtry, tune.parameters = tune.parameters,
                                     num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, 
                                     honesty = TRUE, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
                                     clusters = clusters, samples_per_cluster = samples_per_cluster);
       W.hat <- predict(forest.W)$predictions
+    } else if (length(W.hat) == 1) {
+      W.hat <- rep(W.hat, nrow(X))
+    } else if (length(W.hat) != nrow(X)) {
+      stop("W.hat has incorrect length.")
     }
 
     Y.centered = Y - Y.hat
@@ -176,6 +214,7 @@ causal_forest <- function(X, Y, W,
     forest[["Y.hat"]] <- Y.hat
     forest[["W.hat"]] <- W.hat
     forest[["clusters"]] <- clusters
+    forest[["tunable.params"]] <- tunable.params
     
     class(forest) <- c("causal_forest", "grf")
     forest
