@@ -3,7 +3,6 @@
 #include <sstream>
 #include <vector>
 
-
 #include "commons/globals.h"
 #include "Eigen/Sparse"
 #include "forest/ForestPredictors.h"
@@ -16,43 +15,32 @@ Rcpp::List instrumental_train(Rcpp::NumericMatrix input_data,
                               size_t outcome_index,
                               size_t treatment_index,
                               size_t instrument_index,
-                              std::vector<std::string> variable_names,
                               unsigned int mtry,
                               unsigned int num_trees,
-                              bool verbose,
                               unsigned int num_threads,
                               unsigned int min_node_size,
-                              bool sample_with_replacement,
-                              bool keep_inbag,
                               double sample_fraction,
-                              std::vector<size_t> no_split_variables,
                               unsigned int seed,
                               bool honesty,
                               unsigned int ci_group_size,
-                              double split_regularization,
+                              double reduced_form_weight,
                               double alpha,
-                              double lambda,
-                              bool downweight_penalty) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+                              bool imbalance_penalty,
+                              bool stabilize_splits,
+                              std::vector<size_t> clusters,
+                              unsigned int samples_per_cluster) {
+  ForestTrainer trainer = ForestTrainers::instrumental_trainer(
+      outcome_index - 1,
+      treatment_index - 1,
+      instrument_index - 1,
+      reduced_form_weight,
+      stabilize_splits);
 
-  ForestTrainer trainer = lambda > 0
-    ? ForestTrainers::regularized_instrumental_trainer(data,
-                                                       outcome_index - 1,
-                                                       treatment_index - 1,
-                                                       instrument_index - 1,
-                                                       split_regularization,
-                                                       lambda,
-                                                       downweight_penalty)
-    : ForestTrainers::instrumental_trainer(data,
-                                           outcome_index - 1,
-                                           treatment_index - 1,
-                                           instrument_index - 1,
-                                           split_regularization,
-                                           alpha);
-  RcppUtilities::initialize_trainer(trainer, mtry, num_trees, num_threads, min_node_size,
-      sample_with_replacement, sample_fraction, no_split_variables, seed, honesty, ci_group_size);
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
+  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size,
+      honesty, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
 
-  Forest forest = trainer.train(data);
+  Forest forest = trainer.train(data, options);
 
   Rcpp::List result = RcppUtilities::create_forest_object(forest, data);
   delete data;
@@ -63,10 +51,9 @@ Rcpp::List instrumental_train(Rcpp::NumericMatrix input_data,
 Rcpp::List instrumental_predict(Rcpp::List forest_object,
                                 Rcpp::NumericMatrix input_data,
                                 Eigen::SparseMatrix<double> sparse_input_data,
-                                std::vector <std::string> variable_names,
                                 unsigned int num_threads,
                                 unsigned int ci_group_size) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
   Forest forest = RcppUtilities::deserialize_forest(
       forest_object[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
@@ -82,10 +69,9 @@ Rcpp::List instrumental_predict(Rcpp::List forest_object,
 Rcpp::List instrumental_predict_oob(Rcpp::List forest_object,
                                     Rcpp::NumericMatrix input_data,
                                     Eigen::SparseMatrix<double> sparse_input_data,
-                                    std::vector <std::string> variable_names,
                                     unsigned int num_threads,
                                     unsigned int ci_group_size) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
   Forest forest = RcppUtilities::deserialize_forest(
       forest_object[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
