@@ -3,33 +3,12 @@
 
 #include "commons/DefaultData.h"
 #include "commons/SparseData.h"
+#include "forest/ForestOptions.h"
 #include "RcppUtilities.h"
 #include "serialization/ForestSerializer.h"
 
 const std::string RcppUtilities::SERIALIZED_FOREST_KEY = "serialized.forest";
 
-void RcppUtilities::initialize_trainer(ForestTrainer& forest_trainer,
-                                       uint mtry,
-                                       uint num_trees,
-                                       uint num_threads,
-                                       uint min_node_size,
-                                       bool sample_with_replacement,
-                                       double sample_fraction,
-                                       const std::vector<size_t>& no_split_variables,
-                                       uint seed,
-                                       bool honesty,
-                                       uint ci_group_size) {
-  std::string split_select_weights_file = "";
-  std::string case_weights_file = "";
-
-  std::set<size_t> no_split_set(no_split_variables.begin(), no_split_variables.end());
-
-  forest_trainer.init(mtry, num_trees, seed, num_threads, min_node_size,
-                      no_split_set, split_select_weights_file,
-                      sample_with_replacement, case_weights_file,
-                      sample_fraction, honesty, ci_group_size);
-}
-
 Rcpp::List RcppUtilities::create_forest_object(const Forest& forest,
                                                Data* data) {
   Rcpp::List result;
@@ -62,18 +41,17 @@ Forest RcppUtilities::deserialize_forest(Rcpp::RawVector input) {
 }
 
 Data* RcppUtilities::convert_data(Rcpp::NumericMatrix input_data,
-                                  Eigen::SparseMatrix<double>& sparse_input_data,
-                                  const std::vector<std::string>& variable_names) {
+                                  Eigen::SparseMatrix<double>& sparse_input_data) {
   Data* data;
 
   if (input_data.nrow() > 0) {
     size_t num_rows = input_data.nrow();
     size_t num_cols = input_data.ncol();
-    data = new DefaultData(input_data.begin(), variable_names, num_rows, num_cols);
+    data = new DefaultData(input_data.begin(), num_rows, num_cols);
   } else {
     size_t num_rows = sparse_input_data.rows();
     size_t num_cols = sparse_input_data.cols();
-    data = new SparseData(&sparse_input_data, variable_names, num_rows, num_cols);
+    data = new SparseData(&sparse_input_data, num_rows, num_cols);
   }
 
   data->sort();
@@ -84,6 +62,7 @@ Rcpp::List RcppUtilities::create_prediction_object(const std::vector<Prediction>
   Rcpp::List result;
   result.push_back(RcppUtilities::create_prediction_matrix(predictions), "predictions");
   result.push_back(RcppUtilities::create_variance_matrix(predictions), "variance.estimates");
+  result.push_back(RcppUtilities::create_error_matrix(predictions), "debiased.error");
   return result;
 };
 
@@ -120,7 +99,6 @@ Rcpp::NumericMatrix RcppUtilities::create_variance_matrix(const std::vector<Pred
 
   for (size_t i = 0; i < predictions.size(); i++) {
     const std::vector<double>& variance_estimate = predictions[i].get_variance_estimates();
-
     for (size_t j = 0; j < variance_estimate.size(); j++) {
       double value = variance_estimate[j];
       result(i, j) = value;
@@ -128,3 +106,26 @@ Rcpp::NumericMatrix RcppUtilities::create_variance_matrix(const std::vector<Pred
   }
   return result;
 }
+
+Rcpp::NumericMatrix RcppUtilities::create_error_matrix(const std::vector<Prediction>& predictions) {
+  if (predictions.empty()) {
+    return Rcpp::NumericMatrix(0);
+  }
+
+  Prediction first_prediction = predictions.at(0);
+  if (!first_prediction.contains_error_estimates()) {
+    return Rcpp::NumericMatrix(0);
+  }
+
+  size_t prediction_length = first_prediction.size();
+  Rcpp::NumericMatrix result(predictions.size(), prediction_length);
+
+  for (size_t i = 0; i < predictions.size(); i++) {
+    const std::vector<double>& error_estimate = predictions[i].get_error_estimates();
+    for (size_t j = 0; j < error_estimate.size(); j++) {
+      double value = error_estimate[j];
+      result(i, j) = value;
+    }
+  }
+  return result;
+}
